package Bandiere;


import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.*;
import java.util.Scanner;

import javax.swing.JOptionPane;

import java.io.*;

public class ClientListener implements ActionListener {
	
	public static final String Start = "start";
	public static final String Stop = "stop";
	public static final String Connect = "connect";
	public static final String Disconnect = "disconnect";
	
	private ClientFrame frame;
	private Socket sock;
	private Scanner sockScan;
	private PrintWriter sockPw;
	private DownloaderThread downloadthread;
	private boolean connected;
	private boolean transmitting;
	

	public ClientListener(ClientFrame frame) {
		this.frame = frame;
	}
	
	private void setupConnection() throws IOException {
		try {
			String serverAddress = frame.north.ipAddressField.getText();
			int serverPort = Integer.parseInt(frame.north.portField.getText());
			sock = new Socket(serverAddress, serverPort);
			sockScan = new Scanner(sock.getInputStream());
			sockPw = new PrintWriter(new OutputStreamWriter(sock.getOutputStream()));
		} catch (NumberFormatException e) {
			throw new IOException("Formato porta errato");
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		System.out.println(cmd);
		
		// Connect sequence
		if (cmd.equalsIgnoreCase(ClientListener.Connect)) {
			try {
				setupConnection();
				connected = true;
				JOptionPane.showMessageDialog(frame, "Connessione stabilita");
				frame.center.panel1.setBackground(Color.LIGHT_GRAY);
				frame.center.panel2.setBackground(Color.LIGHT_GRAY);
				frame.center.panel3.setBackground(Color.LIGHT_GRAY);
			} catch (IOException ex) {
				JOptionPane.showMessageDialog(frame, "Impossibile connettersi al server: " + ex.getMessage());
				return;
			}
			
		// Download sequence
		} else if (cmd.equalsIgnoreCase(ClientListener.Start)) {
			frame.center.panel1.setBackground(Color.LIGHT_GRAY);
			frame.center.panel2.setBackground(Color.LIGHT_GRAY);
			frame.center.panel3.setBackground(Color.LIGHT_GRAY);
			downloadthread = new DownloaderThread(frame, this, sockScan);
			transmitting = true;
			sockPw.println(Start);
			sockPw.flush();
			Thread tr = new Thread(downloadthread);
			tr.start();
			
		// Stop sequence
		} else if (cmd.equalsIgnoreCase(ClientListener.Stop)) {
			sockPw.println(ClientListener.Stop);
			sockPw.flush();
			downloadthread.stop();
			transmitting = false;
			
		// Disconnect sequence
		} else if (cmd.equalsIgnoreCase(ClientListener.Disconnect)) {
			sockPw.println(ClientListener.Disconnect);
			if (downloadthread != null) {
				downloadthread.stop();
			}
			
			transmitting = false;
			connected = false;
			
			try {
				sockPw.close();
				sockScan.close();
				sock.close();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		frame.updateUI(connected, transmitting);
	}
	
	public void downloadComplete() {
		transmitting = false;
		frame.updateUI(connected, transmitting);
	}
	
	public void disconnected() {
		frame.north.disconnectButton.doClick();
	}

}
